<?php

namespace Drupal\pagerange404\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Returns a 404 instead of the last available page when a Views pagination
 * request is out of range.
 *
 * @package Drupal\pagerange404\EventSubscriber
 */
class PageRange404Subscriber implements EventSubscriberInterface {

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  public RequestStack $requestStack;

  /**
   * Constructs a PageRange404Subscriber object.
   */
  public function __construct(RequestStack $requestStack) {
    $this->requestStack = $requestStack;
  }

  /**
   * Kernel request event handler.
   */
  public function onKernelRequest(RequestEvent $event): void {

    if ($event->isMainRequest()) {
      $request = $this->requestStack->getCurrentRequest();
      $page = $request->query->get('page');

      if (!empty($page)) {
        $path = $request->getPathInfo();

        $pathValidator = \Drupal::service('path.validator');

        // Check if the path is valid and it returns a Url object.
        if ($url = $pathValidator->getUrlIfValid($path)) {
          // Get route parameters.
          $params = $url->getRouteParameters();

          // If there's a 'view_id' parameter, load the view.
          if (isset($params['view_id'])) {
            // Load the view by view ID and display ID.
            $view = \Drupal\views\Views::getView($params['view_id']);
            $args = [];

            foreach ($params as $key => $value) {
              if (preg_match("/^arg_[0-9]+$/", $key)) {
                $args[] = $value;
              }
            }

            if (!empty($params['display_id'])) {
              $view->setDisplay($params['display_id']);
            }

            $view->setArguments($args);
            $view->execute();
            $current_page = $view->getCurrentPage();

            if ($page != $current_page) {
              throw new NotFoundHttpException();
            }
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => ['onKernelRequest', 100]
    ];
  }

}
